'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _restify = require('restify');

var _restify2 = _interopRequireDefault(_restify);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _InMemoryRepository = require('./repository/InMemoryRepository');

var _InMemoryRepository2 = _interopRequireDefault(_InMemoryRepository);

var _FileRepository = require('./repository/FileRepository');

var _FileRepository2 = _interopRequireDefault(_FileRepository);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SimpleRestfulServer = function () {
  function SimpleRestfulServer(options) {
    _classCallCheck(this, SimpleRestfulServer);

    this.options = options;
    this.logger = new _winston2.default.Logger({
      transports: [new _winston2.default.transports.Console({ level: options.logLevel ? options.logLevel : 'info' })]
    });
    this.createServer();
    this.repositoryClasses = {
      'InMemory': _InMemoryRepository2.default,
      'File': _FileRepository2.default
    };
  }

  _createClass(SimpleRestfulServer, [{
    key: 'start',
    value: function start() {
      var _this = this;

      this.server.listen(this.options.port, function () {
        _this.logger.info('Server started at http:localhost:' + _this.options.port);
      });
    }
  }, {
    key: 'stop',
    value: function stop() {
      this.server.close();
    }
  }, {
    key: 'addResource',
    value: function addResource(resourceInfo) {
      var repository = new this.repositoryClasses[resourceInfo.repository](resourceInfo.repositoryOptions);

      this.server.get('/api/' + resourceInfo.name, function (req, res) {
        repository.getAll().then(function (data) {
          return res.send(data);
        }).catch(function (error) {
          return res.send(500, error);
        });
      });
      this.server.get('/api/' + resourceInfo.name + '/:id', function (req, res) {
        var id = parseInt(req.params['id']);
        repository.get(id).then(function (data) {
          return res.send(data);
        }).catch(function (error) {
          return res.send(404, error);
        });
      });
      this.server.post('/api/' + resourceInfo.name, function (req, res) {
        repository.add(req.body).then(function (data) {
          return res.send(201, data);
        }).catch(function (error) {
          return res.send(500, error);
        });
      });
      this.server.put('/api/' + resourceInfo.name + '/:id', function (req, res) {
        var id = parseInt(req.params['id']);
        repository.update(id, req.body).then(function (data) {
          return res.send(data);
        }).catch(function (error) {
          return res.send(500, error);
        });
      });
      this.server.del('/api/' + resourceInfo.name + '/:id', function (req, res) {
        var id = parseInt(req.params['id']);
        repository.delete(id).then(function () {
          return res.send(204);
        }).catch(function (error) {
          return res.send(500, error);
        });
      });
    }
  }, {
    key: 'addRepository',
    value: function addRepository(repositoryName, repositoryClass) {
      this.repositoryClasses[repositoryName] = repositoryClass;
    }
  }, {
    key: 'createServer',
    value: function createServer() {
      this.server = _restify2.default.createServer();
      this.server.use(_restify2.default.bodyParser());
      this.server.get('status', function (req, res) {
        return res.send({ status: "it's aliiive" });
      });
    }
  }]);

  return SimpleRestfulServer;
}();

exports.default = SimpleRestfulServer;